import gradio as gr
import logging
import sys
from .interface import Interface


logger = logging.getLogger(__name__)
logging.basicConfig(stream=sys.stdout, level=logging.INFO)


class Header(Interface):

    def __init__(self, *args, **kwargs) -> None:
        super().__init__()

        self._make_components(*args, **kwargs)


    def _setup_dark_mode_button(self, *args, **kwargs) -> None:
        dark_mode_button: gr.Button = kwargs['dark_mode_button']

        dark_mode_button.click(
            fn=None,
            js='() => { document.body.classList.toggle(\'dark\'); }',
            show_api=False,
        )


    def _setup(self, *args, **kwargs) -> None:
        self._setup_dark_mode_button(*args, **kwargs)


    def _make_components(self, *args, **kwargs) -> None:
        with gr.Row():
            with gr.Column(scale=4):
                gr.Markdown(value='# 🦈 Shirley WebUI')
                gr.Markdown(
                    value='It is just doing some stuff intelligently. This WebUI is built by \
                    [luojiahai](https://luojiahai.com).'
                )
            with gr.Column(scale=1):
                dark_mode_button = gr.Button(value='🌙 Dark Mode (深色模式)')

        self._setup(
            dark_mode_button=dark_mode_button,
        )



import gradio as gr
import logging
import sys
from .interface import Interface


logger = logging.getLogger(__name__)
logging.basicConfig(stream=sys.stdout, level=logging.INFO)


class Footer(Interface):
    
    def __init__(self, *args, **kwargs) -> None:
        super().__init__()

        self._make_components(*args, **kwargs)


    def _make_components(self, *args, **kwargs) -> None:
        pass



import os
import pathlib
import tempfile
from abc import ABC


class Client(ABC):

    def __init__(self, local: bool, *args, **kwargs) -> None:
        self._local = local
        self._tempdir = os.environ.get('GRADIO_TEMP_DIR') or str(pathlib.Path(tempfile.gettempdir()) / 'gradio')


    @property
    def local(self) -> bool:
        return self._local


    @property
    def tempdir(self) -> str:
        return self._tempdir



from abc import ABC, abstractmethod


class Interface(ABC):

    def __init__(self, *args, **kwargs) -> None:
        pass


    @abstractmethod
    def _make_components(self, *args, **kwargs) -> None:
        raise NotImplementedError()



import gradio as gr
import shirley as sh


def main() -> None:
    avatar_images=(
        sh.utils.getpath('./static/images/grinning-face.png'),
        sh.utils.getpath('./static/images/shark.png'),
    )

    with gr.Blocks(
        theme=gr.themes.Default(
            primary_hue=gr.themes.colors.cyan,
            secondary_hue=gr.themes.colors.sky,
            radius_size=gr.themes.sizes.radius_none,
        ),
        title='Shirley WebUI',
        css=sh.utils.getpath('./static/css/custom.css'),
    ) as blocks:
        sh.interfaces.Header()
        with gr.Tab('✨ Chat (聊天/唠嗑)'):
            sh.interfaces.Chat(avatar_images=avatar_images)
        with gr.Tab('💬 Text-To-Speech (文字转语音)'):
            sh.interfaces.TextToSpeech()
        sh.interfaces.Footer()

    blocks.queue().launch(
        server_name='127.0.0.1',
        server_port=8000,
        favicon_path=sh.utils.getpath('./static/favicon.ico'),
        show_api=False,
    )


if __name__ == '__main__':
    main()



from shirley.utils.pickleablegenerator import PickleableGenerator

import os
import re
from PIL import Image


def getpath(filepath: str) -> str:
    return os.path.abspath(os.path.expanduser(filepath))


def isimage(filepath: str) -> bool:
    try:
        with Image.open(filepath) as img:
            img.verify()
            return True
    except (IOError, SyntaxError):
        return False


def parse(text: str, remove_image_tags: bool = False) -> str:
    lines = text.split('\n')
    lines = [line for line in lines if line != '']
    count = 0
    for i, line in enumerate(lines):
        if '```' in line:
            count += 1
            items = line.split('`')
            if count % 2 == 1:
                lines[i] = f'<pre><code class=\'language-{items[-1]}\'>'
            else:
                lines[i] = f'<br></code></pre>'
        else:
            if i > 0:
                if count % 2 == 1:
                    line = line.replace('`', r'\`')
                    line = line.replace('<', '&lt;')
                    line = line.replace('>', '&gt;')
                    line = line.replace(' ', '&nbsp;')
                    line = line.replace('*', '&ast;')
                    line = line.replace('_', '&lowbar;')
                    line = line.replace('-', '&#45;')
                    line = line.replace('.', '&#46;')
                    line = line.replace('!', '&#33;')
                    line = line.replace('(', '&#40;')
                    line = line.replace(')', '&#41;')
                    line = line.replace('$', '&#36;')
                lines[i] = '<br>' + line

    text = ''.join(lines)
    if remove_image_tags:
        text = text.replace('<ref>', '').replace('</ref>', '')
        text = re.sub(r'<box>.*?(</box>|$)', '', text)
    return text



class PickleableGenerator:

    def __init__(self, fn, *args, **kwargs):
        self.fn = fn
        self.args = args
        self.kwargs = kwargs
        self.generator = fn(*args, **kwargs)
        self.index = 0

    def __iter__(self):
        return self

    def __next__(self):
        try:
            value = next(self.generator)
            self.index += 1
            return value
        except StopIteration:
            raise

    def __getstate__(self):
        return (self.fn, self.args, self.kwargs, self.index)

    def __setstate__(self, state):
        self.fn, self.args, self.kwargs, self.index = state
        self.generator = self.fn(*self.args, **self.kwargs)
        for _ in range(self.index):
            next(self.generator)
